@@ -1,5 +1,5 @@
 /******************************************************************************
-* Copyright 2015-2021 Xilinx, Inc.
+* Copyright 2015-2022 Xilinx, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -245,7 +245,6 @@ AuthenticationAlgorithm* VersalAuthenticationContext::GetAuthenticationAlgorithm
 }
 
 /******************************************************************************/
-
 uint32_t VersalAuthenticationContext::GetCertificateSize(void)
 { 
     return certSize; 
@@ -902,7 +901,7 @@ void VersalAuthenticationContext::CopyPartitionSignature(BootImage& bi, std::lis
     size_t hashSecLen = (*section)->Length;
 
     //If !bootloader AND !headers
-    if ((*section)->firstChunkSize != 0 && !(*section)->isBootloader && (*section)->isPartitionData)
+    if ((*section)->firstChunkSize != 0 && !((*section)->isBootloader && !bi.options.IsVersalNetSeries()) && (*section)->isPartitionData)
     {
         hashSecLen = (*section)->firstChunkSize + hashLength;
     }
@@ -917,7 +916,7 @@ void VersalAuthenticationContext::CopyPartitionSignature(BootImage& bi, std::lis
     memcpy(partitionAc + acSection->Length - signatureLength, (*section)->Data, hashSecLen);
 
     /* Calculate the final hash */
-    Versalcrypto_hash(shaHash, partitionAc, hashSecLen + (acSection->Length - signatureLength), !(*section)->isBootloader);
+    Versalcrypto_hash(shaHash, partitionAc, hashSecLen + (acSection->Length - signatureLength), !((*section)->isBootloader && !bi.options.IsVersalNetSeries()));
     LOG_TRACE("Hash of %s (LE):", acSection->Name.c_str());
     LOG_DUMP_BYTES(shaHash, hashLength);
     /* Create the PKCS padding for the hash */
@@ -1437,7 +1436,7 @@ void static SetDeviceDNA(const uint8_t* dnaValue, uint32_t* deviceDNA)
 {
     for (uint32_t index = 0; index < WORDS_PER_DEVICE_DNA; index++)
     {
-        deviceDNA[index] = ReadBigEndian32(dnaValue);
+        deviceDNA[WORDS_PER_DEVICE_DNA - (index+1)] = ReadBigEndian32(dnaValue);
         dnaValue += sizeof(uint32_t);
     }
 }
@@ -1600,4 +1599,4 @@ void VersalAuthenticationContext::SetKeyLength(Authentication::Type type)
     {
         //AuthenticationContext::authKeyLength = EC_P521_KEY_LENGTH;
     }
-}
\ No newline at end of file
+}
