@@ -1,5 +1,5 @@
 /******************************************************************************
-* Copyright 2015-2021 Xilinx, Inc.
+* Copyright 2015-2022 Xilinx, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -470,7 +470,7 @@ void Key::WritePemFile(std::string filename, RSA * rsa, EC_KEY* eckey, bool secr
             }
             fclose(file);
         }
-        if (eckey != NULL)
+        else if (eckey != NULL)
         {
             if (!secret)
             {
@@ -488,6 +488,10 @@ void Key::WritePemFile(std::string filename, RSA * rsa, EC_KEY* eckey, bool secr
             }
             fclose(file);
         }
+        else
+        {
+            LOG_ERROR("Failed to generate authentication keys for the given algorithm.");
+        }
     }
 
     if (!fileWritten)
