@@ -1,5 +1,5 @@
 /******************************************************************************
-* Copyright 2015-2021 Xilinx, Inc.
+* Copyright 2015-2022 Xilinx, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -64,6 +64,7 @@ BifOptions::BifOptions(Arch::Type architecture, const char* name)
     , pufMode(PufMode::PUF12K)
     , optKey(OptKey::None)
     , shutterVal(SHUTTER_VAL)
+    , pufRoSwapConfigVal(0)
     , dpaCM(DpaCM::DpaCMDisable)
     , ppkSelect(0)
     , spkSelect(1)
@@ -92,6 +93,9 @@ BifOptions::BifOptions(Arch::Type architecture, const char* name)
     , slrConfigCnt(0)
     , aHwrot(false)
     , sHwrot(false)
+    , pmcdataBlocks(0)
+    , dice(DICE::DiceDisable)
+    , slrNum(0xFF)
 {
     arch = architecture;
     SetGroupName(name);
@@ -166,7 +170,12 @@ PartitionBifOptions::PartitionBifOptions()
     , revokeId(0x00000000)
     , slrNum(0xFF)
     , pufHdLoc(PufHdLoc::PUFinEFuse)
+    , versalNetSeries(false)
+    , clusterNum(0)
+    , lockstep(false)
     , updateReserveInPh(false)
+    , bifSection("")
+    , delayAuth(false)
 { }
 
 /******************************************************************************/
@@ -264,6 +273,9 @@ void BifOptions::Add(PartitionBifOptions* currentPartitionBifOptions, ImageBifOp
         {
             pmcDataAesFile = StringUtils::RemoveExtension(StringUtils::BaseName((currentPartitionBifOptions)->filename)) + ".nky";
         }
+
+        if(!(currentPartitionBifOptions->pmcData))
+            partitionBifOptionList.push_back(currentPartitionBifOptions);
         currentPartitionBifOptions->pmcData = true;
         break;
 
@@ -390,7 +402,7 @@ void BifOptions::SetRegInitFileName(std::string filename)
     std::ifstream f(filename.c_str());
     if (!f) 
     {
-        LOG_ERROR("Cannot read file - %s", regInitFile.c_str());
+        LOG_ERROR("Cannot read file - %s", filename.c_str());
     }
     regInitFile = filename;
     LOG_TRACE("Setting Register initialization file as %s", regInitFile.c_str());
@@ -420,7 +432,7 @@ void BifOptions::SetUdfBHFileName(std::string filename)
     std::ifstream f(filename.c_str());
     if (!f)
     {
-        LOG_ERROR("Cannot read file - %s", regInitFile.c_str());
+        LOG_ERROR("Cannot read file - %s", filename.c_str());
     }
     udfBhFile = filename;
     LOG_TRACE("Setting UDF of BH as %s", udfBhFile.c_str());
@@ -471,6 +483,12 @@ void BifOptions::SetPmcdataFile(const std::string& filename)
     pmcdataFile = filename;
 }
 
+/******************************************************************************/
+void BifOptions::ClearPmcCdoFileList()
+{
+    pmcCdoFileList.clear();
+}
+
 /******************************************************************************/
 void BifOptions::SetPmcCdoFileList(const std::string& filename)
 {
@@ -593,6 +611,18 @@ void BifOptions::SetShutterValue(uint32_t value)
     }
 }
 
+/******************************************************************************/
+void BifOptions::SetPufRingOscilltorSwapConfigValue(uint32_t value)
+{
+    pufRoSwapConfigVal = value;
+}
+
+/******************************************************************************/
+void BifOptions::SetDiceEnable()
+{
+    dice = DICE::DiceEnable;
+}
+
 /******************************************************************************/
 void BifOptions::SetPUFHelperFileName(std::string filename)
 {
@@ -1007,12 +1037,15 @@ void BifOptions::CheckForBadKeyandKeySrcPair(std::vector<std::pair<KeySource::Ty
 }
 
 /******************************************************************************/
-static void ValidateEncryptionKeySource(KeySource::Type type)
+static void ValidateEncryptionKeySource(KeySource::Type type, bool versalNetSeries)
 {
-    if ((type == KeySource::EfuseGryKey) || (type == KeySource::BhGryKey) || (type == KeySource::BbramGryKey)
-        || (type == KeySource::EfuseUserGryKey0) || (type == KeySource::EfuseUserGryKey1))
+    if (!versalNetSeries)
     {
-        LOG_ERROR("The usage of obfuscated keys is deprecated in Versal.\n\t   Refer 'bootgen -arch versal -bif_help keysrc' for valid key sources.");
+        if ((type == KeySource::EfuseGryKey) || (type == KeySource::BhGryKey) || (type == KeySource::BbramGryKey)
+            || (type == KeySource::EfuseUserGryKey0) || (type == KeySource::EfuseUserGryKey1))
+        {
+            LOG_ERROR("The usage of obfuscated keys is deprecated in Versal.\n\t   Refer 'bootgen -arch versal -bif_help keysrc' for valid key sources.");
+        }
     }
 
     static bool bhBlkKek = false;
@@ -1091,9 +1124,9 @@ static void ValidateEncryptionKeySource(KeySource::Type type)
 }
 
 /******************************************************************************/
-void BifOptions::SetMetaHeaderEncryptionKeySource(KeySource::Type type)
+void BifOptions::SetMetaHeaderEncryptionKeySource(KeySource::Type type, bool versalNetSeries)
 {
-    ValidateEncryptionKeySource(type);
+    ValidateEncryptionKeySource(type, versalNetSeries);
     metaHdrAttributes.encrKeySource = type;
 }
 
@@ -1162,14 +1195,15 @@ void BifOptions::SetAuthJtagTimeOut(uint32_t value)
 /******************************************************************************/
 void PartitionBifOptions::SetEncryptionKeySource(KeySource::Type type)
 {
-    ValidateEncryptionKeySource(type);
+    ValidateEncryptionKeySource(type, versalNetSeries);
     keySrc = type;
 }
 
 /******************************************************************************/
-void PartitionBifOptions::SetArchType(Arch::Type type)
+void PartitionBifOptions::SetArchType(Arch::Type type, bool flag)
 {
     arch = type;
+    versalNetSeries = flag;
 }
 
 /******************************************************************************/
@@ -1375,7 +1409,7 @@ void PartitionBifOptions::SetHivec(bool flag)
 /******************************************************************************/
 void PartitionBifOptions::SetRevokeId(uint32_t id)
 {
-    if (revokeId > 0xFF)
+    if (id > 0xFF)
     {
         LOG_ERROR("revoke_id can only take values from 0x0 to 0xFF.");
     }
@@ -1388,6 +1422,36 @@ void PartitionBifOptions::SetSlrNum(uint8_t id)
     slrNum = id;
 }
 
+/******************************************************************************/
+void PartitionBifOptions::SetClusterNum(uint8_t id)
+{
+    if (arch == Arch::VERSAL && versalNetSeries)
+    {
+        if (id != 0 && id != 1 && id != 2 && id != 3)
+        {
+            LOG_ERROR("cluster can only take values from 0 to 3.");
+        }
+        clusterNum = id;
+    }
+    else
+    {
+        LOG_ERROR("BIF attribute error !!!\n\t  'cluster' is supported only in VersalNet architecture");
+    }
+}
+
+/******************************************************************************/
+void PartitionBifOptions::SetLockStepFlag()
+{
+    if (arch == Arch::VERSAL && versalNetSeries)
+    {
+        lockstep = true;
+    }
+    else
+    {
+        LOG_ERROR("BIF attribute error !!!\n\t  'lockstep' is supported only in VersalNet architecture");
+    }
+}
+
 /******************************************************************************/
 std::vector<uint32_t>& PartitionBifOptions::GetEncryptionBlocks(void)
 {
@@ -1436,6 +1500,12 @@ void PartitionBifOptions::SetReserveLength(uint64_t length, bool flag)
     }
 }
 
+/******************************************************************************/
+void PartitionBifOptions::SetDelayAuth(bool flag)
+{
+    delayAuth = flag;
+}
+
 /******************************************************************************/
 uint32_t PartitionBifOptions::GetDefaultEncryptionBlockSize(void)
 {
@@ -1467,9 +1537,13 @@ PufHdLoc::Type PartitionBifOptions::GetPufHdLocation(void)
 }
 
 /******************************************************************************/
-std::string PartitionBifOptions::GetOutputFileFromBifSection(std::string out_file, std::string bif_section)
+std::string PartitionBifOptions::GetOutputFileFromBifSection(std::string out_file, std::string bif_section, PartitionType::Type part_type)
 {
     std::string filename = StringUtils::RemoveExtension(out_file) + "_" + bif_section + StringUtils::GetExtension(out_file);
+    if (part_type == PartitionType::SLR_BOOT || part_type == PartitionType::SLR_CONFIG)
+    {
+        filename = StringUtils::RemoveExtension(out_file) + "_" + bif_section + ".bin";
+    }
     return filename;
 }
 
@@ -1746,6 +1820,18 @@ uint32_t BifOptions::GetShutterValue(void)
     return shutterVal;
 }
 
+/******************************************************************************/
+uint32_t BifOptions::GetPufRingOscilltorSwapConfigValue(void)
+{
+    return pufRoSwapConfigVal;
+}
+
+/******************************************************************************/
+DICE::Type BifOptions::GetDice(void)
+{
+    return dice;
+}
+
 /******************************************************************************/
 DpaCM::Type BifOptions::GetDpaCM(void)
 {
@@ -1800,6 +1886,12 @@ std::string BifOptions::GetEfuseUserKek1IVFile(void)
     return efuseUserKek1IVFile;
 }
 
+/******************************************************************************/
+std::string BifOptions::GetUserKeysFileName(void)
+{
+    return userKeyFile;
+}
+
 /******************************************************************************/
 std::string BifOptions::GetUdfBhFile(void)
 {
@@ -1864,6 +1956,12 @@ void BifOptions::SetExtendedIdCode(uint32_t id)
     extendedIdCode = id;
 }
 
+/******************************************************************************/
+void BifOptions::SetPdiType(PartitionType::Type type)
+{
+    pdiType = type;
+}
+
 /******************************************************************************/
 void BifOptions::SetBypassIdcodeFlag(bool flag)
 {
