@@ -1,5 +1,5 @@
 /******************************************************************************
-* Copyright 2015-2021 Xilinx, Inc.
+* Copyright 2015-2022 Xilinx, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -327,7 +327,7 @@ void ZynqMpBootImage::ParseBootImage(PartitionBifOptions* it)
             if (imageAuthBlock != 0)
             {
                 acSize = encrPartLen / (imageAuthBlock * 1024 * 1024);
-                if (encrPartLen % imageAuthBlock != 0)
+                if (encrPartLen % (imageAuthBlock * 1024 * 1024) != 0)
                 {
                     acSize++;
                 }
