@@ -1,5 +1,5 @@
 /******************************************************************************
-* Copyright 2019-2021 Xilinx, Inc.
+* Copyright 2019-2022 Xilinx, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -19,10 +19,14 @@
 
 #include <stdlib.h>
 #include "cdo-command.h"
+#define MARKER_DEVICE      0x1
+#define MARKER_SLRID       0x5
+#define MARKER_DATE        0x6
 
-CDOUTIL_EXPORT void * file_to_buf(const char * path, size_t * sizep);
-CDOUTIL_EXPORT CdoSequence * cdoseq_load_cdo(const char * path);
+void * file_to_buf(const char * path, size_t * sizep);
+CdoSequence * cdoseq_load_cdo(const char * path);
 
-CDOUTIL_EXPORT void cdoseq_extract_writes(CdoSequence * seq);
-CDOUTIL_EXPORT CdoSequence* cdoseq_extract_cdo_till_ssit_sync_slaves(CdoSequence * seq, uint32_t sync_index);
+char slr_id_from_binary(char ch);
+void cdoseq_extract_writes(CdoSequence * seq);
+CdoSequence* cdoseq_extract_cdo_till_ssit_sync_slaves(CdoSequence * seq, uint32_t sync_index);
 #endif /* D_cdo_load */
