@@ -1,5 +1,5 @@
 /******************************************************************************
-* Copyright 2019-2021 Xilinx, Inc.
+* Copyright 2019-2022 Xilinx, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -21,6 +21,10 @@
 #include <inttypes.h>
 #include "cdo-load.h"
 #include "cdo-source.h"
+#include <stdbool.h>
+
+static char* slr_id_ptr;
+static char slr_id;
 
 #if defined(_WIN32)
 #define strcasecmp(x,y) stricmp(x,y)
@@ -71,6 +75,10 @@ struct command_info {
     { "begin", CdoCmdBegin },
     { "end", CdoCmdEnd },
     { "break", CdoCmdBreak },
+    { "ot_check", CdoCmdOtCheck },
+    { "psm_sequence", CdoCmdPsmSequence },
+    { "scatter_write", CdoCmdScatterWrite },
+    { "scatter_write2", CdoCmdScatterWrite2 },
     { "npi_seq", CdoCmdNpiSeq },
     { "npi_precfg", CdoCmdNpiPreCfg },
     { "npi_write", CdoCmdNpiWrite },
@@ -123,6 +131,12 @@ struct command_info {
     { "pm_init_node", CdoCmdPmInitNode },
     { "pm_feature_check", CdoCmdPmFeatureCheck },
     { "pm_iso_control", CdoCmdPmIsoControl },
+    { "pm_bisr", CdoCmdPmBisr },
+    { "pm_apply_trim", CdoCmdPmApplyTrim },
+    { "pm_noc_clock_enable", CdoCmdPmNocClockEnable },
+    { "pm_if_noc_clock_enable", CdoCmdPmIfNocClockEnable },
+    { "pm_activate_subsystem", CdoCmdPmActivateSubsystem },
+    { "pm_set_node_access", CdoCmdPmSetNodeAccess },
     { "cfu_set_crc32", CdoCmdCfuSetCrc32 },
     { "cfu_decompress", CdoCmdCfuDecompress },
     { "cfu_cram_rw", CdoCmdCfuCramRW },
@@ -134,9 +148,20 @@ struct command_info {
     { "cfu_cfi_type", CdoCmdCfuCfiType },
     { "em_set_action", CdoCmdEmSetAction },
     { "ldr_set_image_info", CdoCmdLdrSetImageInfo },
+    { "cframe_clear_check", CdoCmdLdrCframeClearCheck },
     { NULL, 0 }
 };
 
+char* marker_list[] = {
+    "PMC_DATA",
+    "LPD_DATA",
+    "FPD_DATA",
+    "PMC_DATA_SLR_1",
+    "PMC_DATA_SLR_2",
+    "PMC_DATA_SLR_3"
+};
+int marker_count[sizeof(marker_list) / sizeof(marker_list[0])];
+
 static uint32_t iseol(char ** sp) {
     char * s = *sp;
     skipsp(s);
@@ -248,6 +273,36 @@ error:
     return 1;
 }
 
+char slr_id_from_source(char ch)
+{
+    if (slr_id != 0)
+    {
+        ch = slr_id;
+        slr_id = 0;
+    }
+    return ch;
+}
+
+static void check_redundant_markers(char * marker_string)
+{
+    bool same_marker_found = false;
+    int i = 0;
+    for (; i < (sizeof(marker_list) / sizeof(marker_list[0])); i++)
+    {
+        if (strcasecmp(marker_string, marker_list[i]) == 0)
+        {
+            if (marker_count[i]++ > 2)
+            {
+                same_marker_found = true;
+            }
+        }
+    }
+    if (same_marker_found == true)
+    {
+        printf("[WARNING]: The marker %s is found more than once.\n", marker_string);
+    }
+}
+
 CdoSequence * cdoseq_from_source(FILE * f) {
     CdoSequence * seq = cdocmd_create_sequence();
     uint32_t cap = 2;
@@ -277,7 +332,7 @@ CdoSequence * cdoseq_from_source(FILE * f) {
             if (*s == '#') {
                 s++;
                 skipsp(s);
-                cdocmd_add_comment(seq, s);
+                cdocmd_add_comment(seq, "%s", s);
             }
             continue;
         }
@@ -522,6 +577,16 @@ CdoSequence * cdoseq_from_source(FILE * f) {
             if (parse_u32(&s, &value)) goto syntax_error;
             if (iseol(&s)) goto syntax_error;
             if (parse_string(&s, &name)) goto syntax_error;
+            check_redundant_markers(name);
+            if (value == MARKER_DEVICE || value == MARKER_DATE)
+            {
+                break;
+            }
+            if (value == MARKER_SLRID)
+            {
+                slr_id_ptr = name;
+                slr_id = *slr_id_ptr;
+            }
             cdocmd_add_marker(seq, value, name);
             free(name);
             break;
@@ -557,6 +622,40 @@ CdoSequence * cdoseq_from_source(FILE * f) {
             cdocmd_add_break(seq, value);
             break;
         }
+        case CdoCmdOtCheck: {
+            uint32_t value;
+            skipsp(s);
+            if (parse_u32(&s, &value)) goto syntax_error;
+            cdocmd_add_ot_check(seq, value);
+            break;
+        }
+        case CdoCmdPsmSequence:
+            cdocmd_add_psm_sequence(seq);
+            level++;
+            break;
+        case CdoCmdScatterWrite: {
+            uint32_t * buf;
+            uint32_t count;
+            uint32_t value;
+            if (parse_u32(&s, &value)) goto syntax_error;
+            if (parse_buf(&s, &buf, &count)) goto syntax_error;
+            cdocmd_add_scatter_write(seq, value, count, buf, is_be_host());
+            free(buf);
+            break;
+        }
+        case CdoCmdScatterWrite2: {
+            uint32_t * buf;
+            uint32_t count;
+            uint32_t value1;
+            uint32_t value2;
+            if (parse_u32(&s, &value1)) goto syntax_error;
+            if (parse_u32(&s, &value2)) goto syntax_error;
+            if (parse_buf(&s, &buf, &count)) goto syntax_error;
+            cdocmd_add_scatter_write2(seq, value1, value2, count, buf, is_be_host());
+            free(buf);
+            break;
+        }
+
         case CdoCmdNpiSeq:
         case CdoCmdNpiPreCfg:
         case CdoCmdNpiShutdown: {
@@ -981,6 +1080,59 @@ CdoSequence * cdoseq_from_source(FILE * f) {
             cdocmd_add_pm_iso_control(seq, nodeid, value);
             break;
         }
+        case CdoCmdPmBisr: {
+            uint32_t tagid;
+            if (parse_u32(&s, &tagid)) goto syntax_error;
+            cdocmd_add_pm_bisr(seq, tagid);
+            break;
+        }
+
+        case CdoCmdPmApplyTrim: {
+            uint32_t trimtype;
+            if (parse_u32(&s, &trimtype)) goto syntax_error;
+            cdocmd_add_pm_apply_trim(seq, trimtype);
+            break;
+        }
+
+        case CdoCmdPmNocClockEnable: {
+            uint32_t nodeid;
+            uint32_t count;
+            uint32_t * buf;
+            if (parse_u32(&s, &nodeid)) goto syntax_error;
+            if (parse_buf(&s, &buf, &count)) goto syntax_error;
+            cdocmd_add_pm_noc_clock_enable(seq, nodeid, count, buf, is_be_host());
+            free(buf);
+            break;
+        }
+
+        case CdoCmdPmIfNocClockEnable: {
+            uint32_t index;
+            uint32_t state;
+            uint32_t level = 1;
+            if (parse_u32(&s, &index)) goto syntax_error;
+            if (parse_u32(&s, &state)) goto syntax_error;
+            skipsp(s);
+            if (istok(*s) && parse_u32(&s, &level)) goto syntax_error;
+            cdocmd_add_pm_if_noc_clock_enable(seq, index, state, level);
+            break;
+        }
+
+        case CdoCmdPmActivateSubsystem: {
+            uint32_t id;
+            if (parse_u32(&s, &id)) goto syntax_error;
+            cdocmd_add_pm_activate_subsystem(seq, id);
+            break;
+        }
+        case CdoCmdPmSetNodeAccess: {
+            uint32_t id;
+            uint32_t count;
+            uint32_t * buf;
+            if (parse_u32(&s, &id)) goto syntax_error;
+            if (parse_buf(&s, &buf, &count)) goto syntax_error;
+            cdocmd_add_pm_set_node_access(seq, id, count, buf, is_be_host());
+            free(buf);
+            break;
+        }
         case CdoCmdCfuSetCrc32: {
             uint32_t flags;
             uint32_t value;
@@ -1063,6 +1215,12 @@ CdoSequence * cdoseq_from_source(FILE * f) {
             cdocmd_add_ldr_set_image_info(seq, nodeid, uid, puid, funcid);
             break;
         }
+        case CdoCmdLdrCframeClearCheck: {
+            uint32_t id;
+            if (parse_u32(&s, &id)) goto syntax_error;
+            cdocmd_add_ldr_cframe_clear_check(seq, id);
+            break;
+        }
         default:
             goto syntax_error;
         }
@@ -1308,6 +1466,31 @@ void cdoseq_to_source(FILE * f, CdoSequence * seq) {
             }
             fprintf(f, "\n");
             break;
+        case CdoCmdOtCheck:
+            fprintf(f, "ot_check");
+            fprintf(f, " ");
+            print_x64(f, cmd->value);
+            fprintf(f, "\n");
+            break;
+        case CdoCmdPsmSequence:
+            fprintf(f, "psm_sequence\n");
+            level++;
+            break;
+        case CdoCmdScatterWrite:
+            fprintf(f, "scatter_write ");
+            print_x64(f, cmd->value);
+            print_buf(f, cmd->buf, cmd->count);
+            fprintf(f, "\n");
+            break;
+        case CdoCmdScatterWrite2:
+            fprintf(f, "scatter_write2 ");
+            print_x64(f, cmd->value);
+            fprintf(f, " ");
+            print_x64(f, cmd->mask);
+            print_buf(f, cmd->buf, cmd->count);
+            fprintf(f, "\n");
+            break;
+
         case CdoCmdNpiSeq:
             fprintf(f, "npi_seq ");
             print_x64(f, cmd->dstaddr);
@@ -1664,6 +1847,44 @@ void cdoseq_to_source(FILE * f, CdoSequence * seq) {
             print_x64(f, cmd->value);
             fprintf(f, "\n");
             break;
+        case CdoCmdPmBisr:
+            fprintf(f, "pm_bisr ");
+            print_x64(f, cmd->id);
+            fprintf(f, "\n");
+            break;
+        case CdoCmdPmApplyTrim:
+            fprintf(f, "pm_apply_trim ");
+            print_x64(f, cmd->id);
+            fprintf(f, "\n");
+            break;
+        case CdoCmdPmNocClockEnable:
+            fprintf(f, "pm_noc_clock_enable ");
+            print_x64(f, cmd->id);
+            print_buf(f, cmd->buf, cmd->count);
+            fprintf(f, "\n");
+            break;
+        case CdoCmdPmIfNocClockEnable:
+            fprintf(f, "pm_if_noc_clock_enable ");
+            print_x64(f, cmd->id);
+            fprintf(f, " ");
+            print_x64(f, cmd->value);
+	    if (cmd->flags != 1) {
+                fprintf(f, " ");
+                print_x64(f, cmd->flags);
+	    }
+            fprintf(f, "\n");
+            break;
+        case CdoCmdPmActivateSubsystem:
+            fprintf(f, "pm_activate_subsystem ");
+            print_x64(f, cmd->id);
+            fprintf(f, "\n");
+            break;
+        case CdoCmdPmSetNodeAccess:
+            fprintf(f, "pm_set_node_access ");
+            print_x64(f, cmd->id);
+            print_buf(f, cmd->buf, cmd->count);
+            fprintf(f, "\n");
+            break;
         case CdoCmdCfuSetCrc32:
             fprintf(f, "cfu_set_crc32 ");
             print_x64(f, cmd->flags);
@@ -1733,6 +1954,11 @@ void cdoseq_to_source(FILE * f, CdoSequence * seq) {
             print_x64(f, cmd->count);
             fprintf(f, "\n");
             break;
+	case CdoCmdLdrCframeClearCheck:
+            fprintf(f, "cframe_clear_check ");
+            print_x64(f, cmd->id);
+            fprintf(f, "\n");
+            break;
         default:
             fprintf(f, "unknown command (%u)\n", cmd->type);
             break;
