@@ -38,14 +38,15 @@ class BifOptions;
 *********************************************   P R E P R O C E S S O R S   ***
 -------------------------------------------------------------------------------
 */
-#define MAX_NUM_PARTITIONS_VERSAL   32
+#define MAX_NUM_PARTITIONS_VERSAL   20
+#define MAX_NUM_IMAGES_VERSAL       10
 
 /* 128MB Linear QSPI range */
 #define LQSPI_BASE_ADDR_VERSAL      (0xC0000000)
 #define LQSPI_SIZE_VERSAL           (0x8000000) 
 #define LQSPI_RANGE_MASK_VERSAL     (0x7FFFFFF)
 
-#define MAX_IHT_RESERVED_VERSAL     9
+#define MAX_IHT_RESERVED_VERSAL     8
 
 #define MAX_SLAVE_SLRS              3
 #define CDO_COMMAND_SIZE            16
@@ -63,6 +64,8 @@ class BifOptions;
 #define SBI_KEYHOLE_SIZE            0x10000
 #define AIE_BASE_ADDR               0x20000000000
 
+#define SSIT_CHUNK_SIZE             0x8000
+
 typedef enum
 {
     vihImageOwnerShift = 3,
@@ -148,9 +151,10 @@ typedef struct
     uint32_t    extendedIdCode;                     // 0x44
     uint32_t    headerAuthCertificateWordOffset;    // 0x48
     uint32_t    metaHdrGreyOrBlackIV[IV_LENGTH];    // 0x4C
-    uint32_t    reserved[MAX_IHT_RESERVED_VERSAL];  // Reserved
+    uint32_t    optionalDataSize;                   // 0x58
+    uint32_t    reserved[MAX_IHT_RESERVED_VERSAL];  // 0x5C - 0x78
     uint32_t    ihtChecksum;                        // 0x7C
-}VersalImageHeaderTableStructure;
+} VersalImageHeaderTableStructure;
 
 typedef struct
 {
@@ -228,6 +232,7 @@ typedef struct
 typedef struct
 {
     CdoCommandHeader header;
+    uint32_t length;
 } CdoCommandNop;
 #define CDO_CMD_NOP_SIZE 4
 
@@ -324,6 +329,9 @@ public:
     void SetHeaderAuthCertificateOffset(uint32_t offset);
     void SetReservedFields(void);
     void SetChecksum(void);
+    void SetOptionalDataSize(void);
+    void SetOptionalData(uint32_t*, uint32_t);
+    void SetXplmModulesData(BootImage& bi, uint32_t*, uint32_t);
 
     uint32_t GetImageHeaderTableVersion(void);
     uint32_t GetPartitionCount(void);
@@ -388,6 +396,7 @@ public:
     void CreateSlrBootPartition(BootImage& bi);
     void CreateSlrConfigPartition(BootImage& bi);
     void ParseCdos(BootImage& bi, std::vector<std::string> filelist, uint8_t**, size_t*, bool);
+    void ParseSlaveSlrConfigCdos(BootImage & bi, std::vector<std::string> filelist, uint8_t **, size_t *, bool);
     //post-processing
     bool PostProcessCdo(const uint8_t* cdo_data, Binary::Length_t cdo_size);
     bool PostProcessCfi(const uint8_t* cdo_data, Binary::Length_t cdo_size);
