@@ -1,5 +1,5 @@
 /******************************************************************************
-* Copyright 2015-2021 Xilinx, Inc.
+* Copyright 2015-2022 Xilinx, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -276,7 +276,7 @@ void ZynqMpImageHeader::CreateElfPartitions(BootImage& bi, ElfFormat* elf, uint8
 
         /* Override the exec and load address, if startup & load are set through BIF attributes */
         Startup.IsSet() ? (exec_addr = Startup.Value()) : (exec_addr = elf->GetStartUpAddress());
-        if (Load.IsSet())
+        if (Load.IsSet() && iprog == 0)
         {
             load_addr = Load.Value();
         }
@@ -459,7 +459,32 @@ void ZynqMpImageHeader::ImportBin(BootImage& bi)
     {
         hdr->execState = A53ExecState::AARCH32;
     }
-    if(updateReserveInPh == true)
+
+    if (Bootloader)
+    {
+	    /* PMUFW + SPL */
+	    std::string pmu_fw = bi.bifOptions->GetPmuFwImageFile();
+	    ByteFile pmu_fw_data(pmu_fw);
+	    Binary::Length_t pmu_size = pmu_fw_data.len;
+	    uint8_t *partition_data = (uint8_t *)malloc(pmu_size);
+	    memcpy(partition_data, pmu_fw_data.bytes, pmu_size);
+
+	    uint8_t pmu_padding = (4 - (pmu_size & 3)) & 3;
+	    pmuFwSize = totalPmuFwSize = pmu_size + pmu_padding;
+
+	    partition_data = (uint8_t *)realloc(partition_data, pmuFwSize + data.len);
+	    memset(partition_data + pmu_size, 0, pmu_padding);
+	    memcpy(partition_data + pmuFwSize, data.bytes, data.len);
+
+	    data.bytes = partition_data;
+	    fsblFwSize = totalFsblFwSize = data.len + ((4 - (data.len & 3)) & 3);
+	    data.len += pmuFwSize;
+
+	    if (Load.IsSet())
+		    hdr->execAddress = Load.Value();
+    }
+
+    if (updateReserveInPh == true)
     {
         if(Reserve.IsSet())
         {
