@@ -1,5 +1,6 @@
+
 /******************************************************************************
-* Copyright 2015-2021 Xilinx, Inc.
+* Copyright 2015-2022 Xilinx, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -22,6 +23,29 @@
 #include "bootheader-versal.h"
 #include "authentication-versal.h"
 
+typedef enum
+{
+    atfHandoffExecStateShift = 0,
+    atfHandoffEndiannessShift = 1,
+    atfHandoffTrustzoneShift = 2,
+    atfHandoffExceptionLevelShift = 3,
+    atfHandoffDestCpuShift = 5,
+} atf_handoff_partition_attributes;
+
+#define FSBL_MAX_PARTITIONS 6
+
+/* Structure corresponding to each partition entry */
+struct atf_handoff_partition {
+    uint64_t entry_point;
+    uint64_t flags;
+};
+
+/* Structure for hand off parameters to ARM Trusted Firmware (ATF) */
+struct atf_handoff_params_struct {
+    uint8_t magic[4];
+    uint32_t num_entries;
+    struct atf_handoff_partition handoff_partition[FSBL_MAX_PARTITIONS];
+} atf_handoff_params ;
 
 /*
 -------------------------------------------------------------------------------
@@ -37,6 +61,8 @@ VersalPartitionHeader::VersalPartitionHeader(ImageHeader* imageheader, int index
     , dpaCM(DpaCM::DpaCMDisable)
     , pufHdLoc(PufHdLoc::PUFinEFuse)
     , kekIvFile("")
+    , lockstep(Lockstep::LockstepDisable)
+    , cluster(0)
 {
     std::string name;
     slr = 0;
@@ -376,7 +402,7 @@ void VersalPartitionHeader::SetPartitionKeySrc(KeySource::Type keyType, BifOptio
             break;
 
         case KeySource::EfuseUserGryKey1:
-            pHTable->partitionKeySource = EFUSE_USER_BLK_KEY1;
+            pHTable->partitionKeySource = EFUSE_USER_GRY_KEY1;
             kekIvFile = bifOptions->GetEfuseUserKek1IVFile();
             if (kekIvFile == "")
             {
@@ -511,6 +537,11 @@ void VersalPartitionHeader::SetPartitionAttributes(void)
     authBlock = imageHeader->GetAuthBlock();
     dpaCM = imageHeader->GetDpacm();
     pufHdLoc = imageHeader->GetPufHdLocation();
+    cluster = imageHeader->GetClusterNum();
+    if (imageHeader->GetLockStepFlag() == true)
+    {
+        lockstep = Lockstep::LockstepEnable;
+    }
 
     if (hivec) 
     {
@@ -577,7 +608,9 @@ void VersalPartitionHeader::SetPartitionAttributes(void)
                                    (endian << vphtEndiannessShift) |
                                    (partitionType << vphtPartitionTypeShift) |
                                    (hivec << vphtHivecShift) |
-                                   (dpaCM << vphtDpaCMShift) ;
+                                   (dpaCM << vphtDpaCMShift) |
+                                   (cluster << vNetphtClusterShift) |
+                                   (lockstep << vNetphtlockStepShift);
 }
 
 /******************************************************************************/
@@ -882,11 +915,32 @@ void VersalPartitionHeaderTable::Build(BootImage & bi, Binary & cache)
 {
     LOG_INFO("Building the Partition Header Table");
 
+    if (!bi.options.IsVersalNetSeries())
+    {
+        if (getenv("BOOTGEN_SKIP_MAX_PARTITIONS_CHECK") == NULL)
+        {
+            if (bi.subSysImageList.size() > MAX_NUM_IMAGES_VERSAL)
+            {
+                LOG_ERROR("The maximum number of images supported for Versal is %d.\n           No. of images found : %d", MAX_NUM_IMAGES_VERSAL, bi.imageList.size());
+            }
+            if (bi.partitionHeaderList.size() > MAX_NUM_PARTITIONS_VERSAL)
+            {
+                LOG_ERROR("The maximum number of partitions supported for Versal is %d.\n           No. of partitions found : %d", MAX_NUM_PARTITIONS_VERSAL, bi.partitionHeaderList.size());
+            }
+        }
+        else
+        {
+            LOG_WARNING("The ENV BOOTGEN_SKIP_MAX_PARTITIONS_CHECK is set. Skipping check for maximum number of partitions/images.");
+        }
+    }
+
     for (std::list<PartitionHeader*>::iterator partHdr = bi.partitionHeaderList.begin(); partHdr != bi.partitionHeaderList.end(); partHdr++)
     {
         (*partHdr)->Build(bi, cache);
     }
 
+    UpdateAtfHandoffParams(bi);
+
     if (bi.partitionHeaderList.size() > 0)
     {
         bi.partitionHeaderTable->firstSection = bi.partitionHeaderList.front()->section;
@@ -937,7 +991,7 @@ void VersalPartitionHeaderTable::Build(BootImage & bi, Binary & cache)
         int32_t defaultEncrBlockSize = bi.options.bifOptions->metaHdrAttributes.defEncrBlockSize;
         Binary::Length_t encrBlocksSize = 0;
         Binary::Length_t encrOverhead = 0;
-        Binary::Length_t secureChunkSize = VersalPartition::GetSecureChunkSize();
+        Binary::Length_t secureChunkSize = bi.GetSecureChunkSize(true);
 
         bi.options.bifOptions->metaHdrAttributes.encrBlocks.clear();
 
@@ -1017,7 +1071,7 @@ void VersalPartitionHeaderTable::Build(BootImage & bi, Binary & cache)
         /* Note that the last block will always be based on the partition length.*/
 
         std::vector<uint32_t> secureChunkEncrBlocks;
-        uint32_t actualSecureChunkSize = VersalPartition::GetSecureChunkSize() - overhead;
+        uint32_t actualSecureChunkSize = bi.GetSecureChunkSize(true) - overhead;
         uint32_t totalKeyRollencrBlocks = EncryptionContext::GetTotalEncryptionBlocks(bi.imageHeaderTable->metaHeaderLength, secureChunkEncrBlocks, actualSecureChunkSize, &lastBlock);
         secureChunkEncrBlocks.clear();
 
@@ -1167,6 +1221,55 @@ void VersalPartitionHeaderTable::ConfigureMetaHdrAuthenticationContext(BootImage
     }
 }
 
+/******************************************************************************/
+void VersalPartitionHeaderTable::UpdateAtfHandoffParams(BootImage & bi)
+{
+    memset(&atf_handoff_params, 0, sizeof(atf_handoff_params_struct));
+    atf_handoff_params.magic[0] = 'X';
+    atf_handoff_params.magic[1] = 'L';
+    atf_handoff_params.magic[2] = 'N';
+    atf_handoff_params.magic[3] = 'X';
+
+    atf_handoff_params.num_entries = 0;
+
+    for (std::list<PartitionHeader*>::iterator partHdr = bi.partitionHeaderList.begin(); partHdr != bi.partitionHeaderList.end(); partHdr++)
+    {
+        DestinationCPU::Type core = (*partHdr)->imageHeader->GetDestCpu();
+        ExceptionLevel::Type exceptionLevel = (*partHdr)->imageHeader->GetExceptionLevel();
+        TrustZone::Type trustzone = (*partHdr)->imageHeader->GetTrustZone();
+
+        //if((core = a72 - 0 or a72 - 1) && ((EL = EL2 && trustzone = non - secure) || (EL = EL1 && trustzone = secure) || (EL = EL1 && trustzone = non - secure)))
+        bool valid_core = ((core == DestinationCPU::A53_0) || (core == DestinationCPU::A53_1));
+        bool valid_entry1 = ((exceptionLevel == ExceptionLevel::EL2) && (trustzone == TrustZone::NonSecure));
+        bool valid_entry2 = (exceptionLevel == ExceptionLevel::EL1);
+
+        if (valid_core && (valid_entry1 || valid_entry2))
+        {
+            atf_handoff_params.handoff_partition[atf_handoff_params.num_entries].entry_point = (*partHdr)->execAddress;
+
+            uint8_t execState = (*partHdr)->execState;
+            uint8_t endian = ((*partHdr)->elfEndianess == Endianness::BigEndian) ? 1 : 0;
+            uint8_t destCpu = (core == DestinationCPU::A53_0) ? 0 : 1;
+
+            atf_handoff_params.handoff_partition[atf_handoff_params.num_entries].flags = (execState << atfHandoffExecStateShift) |
+                (endian << atfHandoffEndiannessShift) |
+                (trustzone << atfHandoffTrustzoneShift) |
+                (exceptionLevel << atfHandoffExceptionLevelShift) |
+                (destCpu << atfHandoffDestCpuShift);
+
+            atf_handoff_params.num_entries++;
+        }
+    }
+
+    for (std::list<PartitionHeader*>::iterator partHdr = bi.partitionHeaderList.begin(); partHdr != bi.partitionHeaderList.end(); partHdr++)
+    {
+        if ((*partHdr)->update_atf_handoff_params)
+        {
+            memcpy((*partHdr)->partition->section->Data + (*partHdr)->atf_handoff_params_offset, &atf_handoff_params, sizeof(atf_handoff_params_struct));
+        }
+    }
+}
+
 /******************************************************************************/
 void VersalPartitionHeaderTable::Link(BootImage & bi)
 {
