@@ -1,5 +1,6 @@
+
 /******************************************************************************
-* Copyright 2015-2021 Xilinx, Inc.
+* Copyright 2015-2022 Xilinx, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -87,6 +88,14 @@ void ZynqMpReadImage::ReadBinaryFile(DumpOption::Type dump, std::string path)
         {
             LOG_ERROR("Error reading Image header table");
         }
+        if ((iHT->version != VERSION_ZYNQ_ZYNQMP))
+        {
+            LOG_ERROR("Improper version (0x%.8x) read from Image Header Table of the boot image file.", iHT->version);
+        }
+        if (!((iHT->partitionTotalCount > 0) && (iHT->partitionTotalCount < 0xFF)))
+        {
+            LOG_ERROR("Number of partitions read is more than number of supported partiiton count.");
+        }
     }
     else
     {
@@ -287,7 +296,6 @@ void ZynqMpReadImage::DisplayImageHeaders(void)
         Separator();
         DisplayValue("next_ih(W) (0x00) : ", (*iH)->nextImageHeaderWordOffset);
         DisplayValue("next_pht(W) (0x04) : ", (*iH)->partitionHeaderWordOffset);
-        DisplayValue("total_partitions (0x08) : ", (*iH)->dataSectionCount);
         DisplayValue("total_partitions (0x0c) : ", (*iH)->imageNameLength);
         DisplayAscii("name (0x10) : ", *name);
     }
